package pokercore

import "errors"

var ErrDuplicateCard = errors.New("cannot score a poker hand out of a set of cards with duplicates")

// this method makes a n new hands where n is the number of cards in the hand
// each of the new hands has one card removed from the original hand
// then it calls the identifyBestFiveCardPokerHand method on each of the new hands
// and returns the best hand by score.  this is recursion.
func (hand Cards) identifyBestFiveCardPokerHand() (Cards, error) {
	if hand.containsDuplicates() {
		return nil, ErrDuplicateCard
	}

	newHands := make([]Cards, len(hand))
	for i := 0; i < len(hand); i++ {
		newHand := hand[:i]
		newHand = append(newHand, hand[i+1:]...)
		newHands[i] = newHand
	}
	var bestHand Cards
	var bestScore HandScore

	for _, h := range newHands {
		if len(h) == 5 {
			score, _ := h.PokerHandScore()
			if score > bestScore {
				bestScore = score
				bestHand = h
			}
		} else {
			rh, _ := h.identifyBestFiveCardPokerHand()
			score, _ := rh.PokerHandScore()
			if score > bestScore {
				bestScore = score
				bestHand = rh
			}
		}
	}
	return bestHand, nil
}
