package pokercore

import "fmt"

type HandScore int

const (
	ScoreHighCard = HandScore(iota * 100_000_000_000)
	ScorePair
	ScoreTwoPair
	ScoreThreeOfAKind
	ScoreStraight
	ScoreFlush
	ScoreFullHouse
	ScoreFourOfAKind
	ScoreStraightFlush
	ScoreRoyalFlush
)

func (c Card) Score() HandScore {
	return HandScore(c.Rank.Score())
}

func (c Cards) PokerHandScore() (HandScore, error) {
	if len(c) != 5 {
		return 0, fmt.Errorf("hand must have 5 cards to be scored as a poker hand")
	}
	ph, err := c.PokerHand()
	if err != nil {
		return 0, err
	}

	fmt.Println(ph)
	return ph.Score, nil
}

/*
func scoreToName(x HandScore) string {
	switch x {
	case x >= ScoreRoyalFlush:
		return "Royal Flush"
	case x >= ScoreStraightFlush:
		return "Straight Flush"
	case x >= ScoreFourOfAKind:
		return "Four of a Kind"
	case x >= ScoreFullHouse:
		return "Full House"
	case x >= ScoreFlush:
		return "Flush"
	case x >= ScoreStraight:
		return "Straight"
	case x >= ScoreThreeOfAKind:
		return "Three of a Kind"
	case x >= ScoreTwoPair:
		return "Two Pair"
	case x >= ScorePair:
		return "Pair"
	case x >= ScoreHighCard:
		return "High Card"
	}
}
*/

func (r Rank) Article() string {
	switch r {
	case ACE:
		return "an"
	case EIGHT:
		return "an"
	default:
		return "a"
	}
}

func (r Rank) WithArticle() string {
	return fmt.Sprintf("%s %s", r.Article(), r)
}

func (r Rank) Pluralize() string {
	switch r {
	case ACE:
		return "Aces"
	case DEUCE:
		return "Deuces"
	case THREE:
		return "Threes"
	case FOUR:
		return "Fours"
	case FIVE:
		return "Fives"
	case SIX:
		return "Sixes"
	case SEVEN:
		return "Sevens"
	case EIGHT:
		return "Eights"
	case NINE:
		return "Nines"
	case TEN:
		return "Tens"
	case JACK:
		return "Jacks"
	case QUEEN:
		return "Queens"
	case KING:
		return "Kings"
	default:
		panic("nope")
	}
}

func (x HandScore) String() string {
	return fmt.Sprintf("<HandScore %d>", x)
	//return scoreToName(x)
}

func (r Rank) Score() HandScore {
	switch r {
	case DEUCE:
		return 2
	case THREE:
		return 3
	case FOUR:
		return 4
	case FIVE:
		return 5
	case SIX:
		return 6
	case SEVEN:
		return 7
	case EIGHT:
		return 8
	case NINE:
		return 9
	case TEN:
		return 10
	case JACK:
		return 11
	case QUEEN:
		return 12
	case KING:
		return 13
	case ACE:
		return 14
	}
	return 0
}
